using Godot;

namespace Rokojori
{  
  public static class ColorX 
  {
    public static float AlphaMultipliedR( this Color c )
    {
      return c.R * c.A;
    }

    public static float AlphaMultipliedG( this Color c )
    {
      return c.G * c.A;
    }

    public static float AlphaMultipliedB( this Color c )
    {
      return c.B * c.A;
    }

    public static Color Blend( Color bottom, Color top )
    {
      var fade = ( 1f - top.A );
      var a0 = top.A + bottom.A * fade;

      var xR = top.AlphaMultipliedR() + bottom.AlphaMultipliedR() * fade;
      var xG = top.AlphaMultipliedG() + bottom.AlphaMultipliedG() * fade;
      var xB = top.AlphaMultipliedB() + bottom.AlphaMultipliedB() * fade;
      
      var normalizer = 1f / a0;

      return new Color( xR, xG, xB, 1 ) * normalizer;
    }

    public static Color Lerp( Color a, Color b, float amount )
    {
      return a.Lerp( b, amount ); 
    }

    public static Vector3 RGB( Color c )
    {
      return new Vector3( c.R, c.G, c.B );
    }

    public static Vector2 RA( Color c )
    {
      return new Vector2( c.R, c.A);
    }

    public static Color Create( Vector3 rgb, float a )
    {
      return new Color( rgb.X, rgb.Y, rgb.Z, a );
    }

    public static Vector4 ToVector4( Color c )
    {
      return new Vector4( c.R, c.G, c.B, c.A );
    }

    public static Color FromVector4( Vector4 c )
    {
      return new Color( c.X, c.Y, c.Z, c.W );
    }

    public static Color Fade( Color color, float fade )
    {
      color.A = color.A * fade;
      return color;
    }

    public static Color FadeAlpha( this Color color, float fade )
    {
      color.A = color.A * fade;
      return color;
    }

    public static Color SetAlpha( Color color, float alpha )
    {
      color.A = alpha;
      return color;
    }
  
  }
}